==> altshift-0.2.0.tgz: package/lib/altshift/jsgi.js <==
/*jslint indent:4 */
/**
 * Imports
 */
var http = require('http');
var core = require('./core');


var Request = core.class('Request', {
    include: [core.ConstantScope],
    extend: {
        jsgi: {
            version: [0, 3],
            multithread: false,
            multiprocess: true,
            async: true,
            runOnce: false,
            errors: {
                print: sys.puts,
                flush: function (){}
            }
        }
        /*get host() {
            var host = this.headers.host;
            return host ? host.split(':')[0] : '';
        },
        get port() {
            var host = this.headers.host;
            return host ? (host.split(":")[1] || 80) : 80;
        },
        get remoteAddr() {
            return this.nodeRequest.connection.remoteAddress;
        },
        get version() {
            return [ this.nodeRequest.httpVersionMajor, this.nodeRequest.httpVersionMinor ]
        }*/
    },

    /**
     * Request constructor
     *
     * @constructor
     */
    initialize: function (options) {
        this.env = options.env || {};
        this.scriptName = options.scriptName || '';
        this.protocol = options.protocol || 'http';
        this.headers = options.headers || {};
        this.method = request.method.toUpperCase() || 'GET';


        this.path = options.path || '';
        this.query = options.query || {};
    },
    finalize: function () {
    },

    getHost: function () {
        var host = this.headers.host;
        return host ? host.split(':')[0] : '';
    },
    getPort: function () {
        var host = this.headers.host;
        return host ? (host.split(':')[1] || 80) : 80;
    }
});

function Request( request ) {
  var url = request.url;
  var questionIndex = url.indexOf('?');

  this.nodeRequest = request;
  if(questionIndex > -1){
    this.pathInfo = url.substring(0, questionIndex);
    this.queryString = url.substring(questionIndex + 1);
  } else {
    this.pathInfo = url;
    this.queryString = "";
  }
  if(this.method !== 'GET'){ // optimize GET
   this.body = new Input( request );
  }

}


function Input( request ) {
  var
    inputBuffer = [],
    waitingForLength = Infinity;
  function callback(data){
    inputBuffer.push(data);
  }
  var deferred = defer();
  request
      .addListener( "data", function( data ) {
         callback(data);
      })
      .addListener( "end", function() {
        deferred.resolve();
      });

  this.forEach = function (each) {
    if (this.encoding) {
      request.setBodyEncoding( this.encoding );
    }
    inputBuffer.forEach(each);
    callback = each;
    return deferred.promise;
  };
}

Input.prototype.join = function(token) {
  var parts = [];
  return this.forEach(function (part) {
    parts.push(part);
  }).then(function(){
    return parts.join(token || ""); // yeah, I know Array.prototype.join defaults to ",", but clearly "" is more useful here
  });
}

function Response( response, stream ) {
  var started = false, canceller, cancel;
  return handle;

  function handle( data, notDone ) {
    var forEachResult;
    if ( typeof data.then === "function" ) {
      if(!canceller){
        canceller = function(){
          cancel && cancel();
        }
        stream.addListener("close", canceller);
      }
      cancel = data.cancel;
      data.then(
        handle,
        function( error ) {
          sys.puts("Error: " + error.stack);
          handle({ status:500, headers:{}, body:[error.message] });
        },
        function( data ){
          handle( data, true);
        }
      );

      return;
    }

    if ( !started ) {
      started = true;
      response.writeHead( data.status, data.headers );
    }

    try {
      if ( typeof data.body === "string" ) {
        response.write(data.body);
      }
      else if ( typeof data.body.forEach !== "function" ) {
        throw new Error("The body does not have a forEach function");
      }

      forEachResult = data.body.forEach( function( chunk ) {
       try{
          response.write( chunk, data.body.encoding || "utf8" );
       }catch(e){
       sys.puts( "error writing " + chunk + e);
       }
      });

      if ( !notDone && forEachResult && ( typeof forEachResult.then === "function" ) ) {
        cancel = forEachResult.cancel;
        forEachResult.then( function() {
          if(canceller){
            stream.addListener("close", canceller);
          }
          response.end();
        });
      }

      else if ( !notDone ) {
        if(canceller){
          stream.addListener("close", canceller);
        }
        response.end();
      }
    }

    catch( e ) {
      if(canceller){
        stream.addListener("close", canceller);
      }
      try{
       // if it is not too late, set the status
       response.writeHead(500, {});
      }catch(e2){}
      try{
          response.write( "Error: " + e.stack );
          response.end();
      }catch(e3){
       sys.puts(e3.stack);
      }
    }
  }
}

function Listener( app ) {
  return function( request, response ) {
   var connection = request.connection;
    request = new Request( request );
    var respond = new Response( response, connection );
    process.nextTick(function(){
      var jsgiResponse;
      try {
        jsgiResponse = app( request )
      } catch( error ) {
        jsgiResponse = { status:500, headers:{}, body:[error.stack] };
      }
      respond( jsgiResponse );
    });
  }
}
start.Request = Request;
start.Listener = Listener;
start.start = start;

function start(app, options) {
  app = new Listener(app);
  options = options || {};

  var port = options.port || 8080;

  var httpServer = http.createServer( app );
  httpServer.listen( port );
  sys.puts( "Server running on port " + port );
  return httpServer;
};
module.exports = start;

==> altshift-0.3.0-2.tgz: package/lib/altshift/jsgi.js <==
/*jslint indent:4 */
/**
 * Imports
 */
var http = require('http');
var core = require('./core');


var Request = core.class('Request', {
    include: [core.ConstantScope],
    extend: {
        jsgi: {
            version: [0, 3],
            multithread: false,
            multiprocess: true,
            async: true,
            runOnce: false,
            errors: {
                print: sys.puts,
                flush: function (){}
            }
        }
        /*get host() {
            var host = this.headers.host;
            return host ? host.split(':')[0] : '';
        },
        get port() {
            var host = this.headers.host;
            return host ? (host.split(":")[1] || 80) : 80;
        },
        get remoteAddr() {
            return this.nodeRequest.connection.remoteAddress;
        },
        get version() {
            return [ this.nodeRequest.httpVersionMajor, this.nodeRequest.httpVersionMinor ]
        }*/
    },

    /**
     * Request constructor
     *
     * @constructor
     */
    initialize: function (options) {
        options = options || {};

        this.env = options.env || {};
        this.scriptName = options.scriptName || '';
        this.scheme = options.scheme || 'http';
        this.headers = options.headers || {};
        this.method = options.method.toUpperCase() || 'GET';


        this.pathInfo = options.pathInfo || '';
        this.queryString = options.queryString || '';

        if (this.method !== 'GET') { // optimize GET
            this.body = new Input(request);
        }
    },
    finalize: function () {
    },

    getHost: function () {
        var host = this.headers.host;
        return host ? host.split(':')[0] : '';
    },
    getPort: function () {
        var host = this.headers.host;
        return host ? (host.split(':')[1] || 80) : 80;
    }
});

function Request( request ) {
  var url = request.url;
  var questionIndex = url.indexOf('?');

  this.nodeRequest = request;
  if(questionIndex > -1){
    this.pathInfo = url.substring(0, questionIndex);
    this.queryString = url.substring(questionIndex + 1);
  } else {
    this.pathInfo = url;
    this.queryString = "";
  }


}


function Input( request ) {
  var
    inputBuffer = [],
    waitingForLength = Infinity;
  function callback(data){
    inputBuffer.push(data);
  }
  var deferred = defer();
  request
      .addListener('data', function( data ) {
         callback(data);
      })
      .addListener('end', function() {
        deferred.resolve();
      });

  this.forEach = function (each) {
    if (this.encoding) {
      request.setBodyEncoding(this.encoding);
    }
    inputBuffer.forEach(each);
    callback = each;
    return deferred.getPromise();
  };
}

Input.prototype.join = function(token) {
  var parts = [];
  return this.forEach(function (part) {
    parts.push(part);
  }).then(function () {
    return parts.join(token || ''); // yeah, I know Array.prototype.join defaults to ",", but clearly "" is more useful here
  });
}

function Response( response, stream ) {
  var started = false, canceller, cancel;
  return handle;

  function handle( data, notDone ) {
    var forEachResult;
    if ( typeof data.then === "function" ) {
      if(!canceller){
        canceller = function(){
          cancel && cancel();
        }
        stream.addListener("close", canceller);
      }
      cancel = data.cancel;
      data.then(
        handle,
        function( error ) {
          sys.puts("Error: " + error.stack);
          handle({ status:500, headers:{}, body:[error.message] });
        },
        function( data ){
          handle( data, true);
        }
      );

      return;
    }

    if ( !started ) {
      started = true;
      response.writeHead( data.status, data.headers );
    }

    try {
      if ( typeof data.body === "string" ) {
        response.write(data.body);
      }
      else if ( typeof data.body.forEach !== "function" ) {
        throw new Error("The body does not have a forEach function");
      }

      forEachResult = data.body.forEach( function( chunk ) {
       try{
          response.write( chunk, data.body.encoding || "utf8" );
       }catch(e){
       sys.puts( "error writing " + chunk + e);
       }
      });

      if ( !notDone && forEachResult && ( typeof forEachResult.then === "function" ) ) {
        cancel = forEachResult.cancel;
        forEachResult.then( function() {
          if(canceller){
            stream.addListener("close", canceller);
          }
          response.end();
        });
      }

      else if ( !notDone ) {
        if(canceller){
          stream.addListener("close", canceller);
        }
        response.end();
      }
    }

    catch( e ) {
      if(canceller){
        stream.addListener("close", canceller);
      }
      try{
       // if it is not too late, set the status
       response.writeHead(500, {});
      }catch(e2){}
      try{
          response.write( "Error: " + e.stack );
          response.end();
      }catch(e3){
       sys.puts(e3.stack);
      }
    }
  }
}

function Listener(app) {
    return function(request, response) {
        var connection = request.connection;
        request = new Request(request);
        var respond = new Response(response, connection);
        process.nextTick(function () {
            var jsgiResponse;
            try {
                jsgiResponse = app(request);
            } catch( error ) {
                jsgiResponse = {
                        status:500,
                        headers:{},
                        body:[error.stack]
                };
            }
            respond(jsgiResponse);
        });
    };
}
start.Request = Request;
start.Listener = Listener;
start.start = start;

function start(app, options) {
  app = new Listener(app);
  options = options || {};

  var port = options.port || 8080;

  var httpServer = http.createServer(app);
  httpServer.listen(port);
  sys.puts( "Server running on port " + port );
  return httpServer;
};
module.exports = start;
